;;; gmsh-mode.el --- Highlight GMSH mesh generator script syntax -*- lexical-binding: t -*-

;; Author: Matsievskiy S.V.
;; Maintainer: Matsievskiy S.V.
;; Package-Version: 20240223.558
;; Package-Revision: 324d09e6ef51
;; Package-Requires: ((emacs "26.1"))
;; Homepage: https://gitlab.com/matsievskiysv/gmsh-mode
;; Keywords: languages


;; This file is not part of GNU Emacs

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3, or (at your option)
;; any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; For a full copy of the GNU General Public License
;; see <http://www.gnu.org/licenses/>.


;;; Commentary:

;; Highlight GMSH mesh generator script syntax.

(require 'cc-mode)

;;; Code:

(defvar gmsh-fontlock-builtin-functions
  (list
   "Acos"
   "Asin"
   "Atan"
   "Atan2"
   "Ceil"
   "Cos"
   "Cosh"
   "Exp"
   "Fabs"
   "Fmod"
   "Floor"
   "Hypot"
   "Log"
   "Log10"
   "Max"
   "Min"
   "Modulo"
   "Rand"
   "Round"
   "Sqrt"
   "Sin"
   "Sinh"
   "Tan"
   "Tanh"
   "Cpu"
   "Memory"
   "TotalMemory"
   "newp"
   "newc"
   "newcl"
   "newl"
   "news"
   "newv"
   "newll"
   "newsl"
   "newreg"
   "Str"
   "DefineConstant"
   "SetNumber"
   "SetString"
   "Abort"
   "Exit"
   "CreateDir"
   "Printf"
   "Warning"
   "Error"
   "Merge"
   "ShapeFromFile"
   "Draw"
   "SplitCurrentWindowHorizontal"
   "SplitCurrentWindowVertical"
   "SetCurrentWindow"
   "UnsplitWindow"
   "SetChanged"
   "BoundingBox"
   "Delete Model"
   "Delete Physicals"
   "Delete Variables"
   "Delete Options"
   "Delete"
   "Print"
   "Sleep"
   "SystemCall"
   "NonBlockingSystemCall"
   "OnelabRun"
   "SetName"
   "SetFactory"
   "SyncModel"
   "NewModel"
   "Include"
   "Physical Point"
   "Point"
   "Line"
   "Bezier"
   "BSpline"
   "Spline"
   "Circle"
   "Ellipse"
   "Compound"
   "Curve Loop"
   "Curve"
   "Wire"
   "Physical Curve"
   "Line Loop"
   "Physical Surface"
   "Surface"
   "Plane Surface"
   "BSpline Surface"
   "Bezier Surface"
   "Disk"
   "Rectangle"
   "Surface Loop"
   "Volume"
   "Sphere"
   "Box"
   "Cylinder"
   "Torus"
   "Cone"
   "Wedge"
   "ThruSections"
   "Ruled ThruSections"
   "Physical Volume"
   "Extrude"
   "Layers"
   "QuadTriNoNewVerts"
   "QuadTriAddVerts"
   "Fillet"
   "Chamfer"
   "BooleanIntersection"
   "BooleanUnion"
   "BooleanDifference"
   "BooleanFragments"
   "Dilate"
   "Rotate"
   "Symmetry"
   "Affine"
   "Translate"
   "Boundary"
   "CombinedBoundary"
   "PointsOf"
   "Intersect Curve"
   "Split Curve"
   "Coherence"
   "Recursive Delete"
   "Delete Embedded"
   "SetMaxTag Point"
   "Recursive Hide"
   "Hide"
   "Recursive Show"
   "Show"
   "MeshSize"
   "Background Field"
   "Field"
   "Attractor"
   "AttractorAnisoCurve"
   "AutomaticMeshSizeField"
   "Ball"
   "BoundaryLayer"
   "Curvature"
   "Distance"
   "ExternalProcess"
   "Frustum"
   "Gradient"
   "IntersectAniso"
   "Laplacian"
   "LonLat"
   "MathEval"
   "MathEvalAniso"
   "MaxEigenHessian"
   "Mean"
   "MinAniso"
   "Octree"
   "Param"
   "PostView"
   "Restrict"
   "Structured"
   "Threshold"
   "Transfinite Curve"
   "Transfinite Surface"
   "Transfinite Volume"
   "TransfQuadTri"
   "RefineMesh"
   "OptimizeMesh"
   "AdaptMesh"
   "RelocateMesh"
   "RecombineMesh"
   "Recombine"
   "SetOrder"
   "PartitionMesh"
   "Periodic Curve"
   "Periodic Surface"
   "Coherence Mesh"
   "CreateTopology"
   "CreateGeometry"
   "ClassifySurfaces"
   "RenumberMeshNodes"
   "RenumberMeshElements"
   "Recursive Color"
   "Recombine Surface"
   "MeshAlgorithm Surface"
   "MeshSizeFromBoundary Surface"
   "Compound Curve"
   "Compaund Surface"
   "ReverseMesh"
   "ReorientMesh"
   "Save"
   "Smoother Surface"
   "Homology"
   "Cohomology"
   "Alias View"
   "AliasWithOptions View"
   "CopyOptions View"
   "Combine ElementsByViewName"
   "Combine ElementsFromAllViews"
   "Combine Views"
   "Combine ElementsFromVisibleViews"
   "Combine TimeStepsByViewName"
   "Combine TimeSteps"
   "Combine TimeStepsFromAllViews"
   "Combine TimeStepsFromVisibleViews"
   "Delete View"
   "Delete Empty Views"
   "Background Mesh View"
   "Plugin"
   "Run"
   "SendToServer View"
   "TIME"
   "INTERPOLATION_SCHEME"
   "Find"
   "StrFind"
   "StrCmp"
   "StrLen"
   "TextAttributes"
   "Exists"
   "FileExists"
   "StringToName"
   "GetNumber"
   "GetValue"
   "DefineNumber"
   "List"
   "Unique"
   "Abs"
   "ListFromFile"
   "LinSpace"
   "LogSpace"
   "Mass"
   "CenterOfMass"
   "MatrixOfInertia"
   "Today"
   "OnelabAction"
   "GmshExecutableName"
   "CurrentDirectory"
   "CurrentDir"
   "CurrentFileName"
   "StrPrefix"
   "StrRelative"
   "StrCat"
   "StrChoice"
   "StrSub"
   "UpperCase"
   "AbsolutePath"
   "DirName"
   "Sprintf"
   "GetEnv"
   "GetString"
   "GetStringValue"
   "StrReplace"
   "NameToString"
   "DefineString"
   "Mesh"
   "Duplicata"
   "Left"
   "Right"
   "Alternate"
   "AlternateRight"
   "AlternateLeft"
   "Using View"
   "ScaleLastLayer"
   "Using Progression"
   "Bump"
   "N2S"
   "System"
   "Using Sewing"
   "Using Wire"
   "Using")
  "List of GMSH builtin functions.")

(defvar gmsh-fontlock-keywords
  (list
   "For"
   "In"
   "EndFor"
   "If"
   "ElseIf"
   "Else"
   "EndIf"
   "Return"
   "Choices"
   "Name"
   "Macro"
   "Call")
  "List of GMSH keywords.")

(defvar gmsh-fontlock-constants
  (list
   "View"
   "Pi"
   "GMSH_MAJOR_VERSION"
   "GMSH_MINOR_VERSION"
   "GMSH_PATCH_VERSION"
   "MPI_Size"
   "MPI_Rank"
   "Mesh"
   "Geometry"
   "DoubleClickedPointCommand"
   "DoubleClickedLineCommand"
   "DoubleClickedCurveCommand"
   "DoubleClickedSurfaceCommand"
   "DoubleClickedVolumeCommand"
   "OCCTargetUnit"
   "AutoCoherence"
   "Clip"
   "CopyMeshingMethod"
   "Curves"
   "CurveLabels"
   "CurveSelectWidth"
   "CurveType"
   "CurveWidth"
   "DoubleClickedEntityTag"
   "ExactExtrusion"
   "ExtrudeReturnLateralEntities"
   "ExtrudeSplinePoints"
   "HighlightOrphans"
   "LabelType"
   "Light"
   "LightTwoSide"
   "MatchGeomAndMesh"
   "MatchMeshScaleFactor"
   "MatchMeshTolerance"
   "Normals"
   "NumSubEdges"
   "OCCAutoEmbed"
   "OCCAutoFix"
   "OCCBooleanPreserveNumbering"
   "OCCBoundsUseStl"
   "OCCDisableStl"
   "OCCFixDegenerated"
   "OCCFixSmallEdges"
   "OCCFixSmallFaces"
   "OCCImportLabels"
   "OCCMakeSolids"
   "OCCParallel"
   "OCCScaling"
   "OCCSewFaces"
   "OCCThruSectionsDegree"
   "OCCUnionUnify"
   "OCCUseGenericClosestPoint"
   "OffsetX"
   "OffsetY"
   "OffsetZ"
   "OldCircle"
   "OldRuledSurface"
   "OldNewReg"
   "OrientedPhysicals"
   "Points"
   "PointLabels"
   "PointSelectSize"
   "PointSize"
   "PointType"
   "ReparamOnFaceRobust"
   "ScalingFactor"
   "SnapPoints"
   "SnapX"
   "SnapY"
   "SnapZ"
   "Surfaces"
   "SurfaceLabels"
   "SurfaceType"
   "Tangents"
   "Tolerance"
   "ToleranceBoolean"
   "Transform"
   "TransformXX"
   "TransformXY"
   "TransformXZ"
   "TransformYX"
   "TransformYY"
   "TransformYZ"
   "TransformZX"
   "TransformZY"
   "TransformZZ"
   "Volumes"
   "VolumeLabels"
   "Color"
   "Selection"
   "HighlightZero"
   "HighlightOne"
   "HighlightTwo"
   "Projection"
   "General"
   "AxesFormatX"
   "AxesFormatY"
   "AxesFormatZ"
   "AxesLabelX"
   "AxesLabelY"
   "AxesLabelZ"
   "BackgroundImageFileName"
   "BuildInfo"
   "BuildOptions"
   "DefaultFileName"
   "Display"
   "ErrorFileName"
   "ExecutableFileName"
   "FileName"
   "FltkTheme"
   "GraphicsFont"
   "GraphicsFontEngine"
   "GraphicsFontTitle"
   "OptionsFileName"
   "RecentFile0"
   "RecentFile1"
   "RecentFile2"
   "RecentFile3"
   "RecentFile4"
   "RecentFile5"
   "RecentFile6"
   "RecentFile7"
   "RecentFile8"
   "RecentFile9"
   "SessionFileName"
   "ScriptingLanguages"
   "TextEditor"
   "TmpFileName"
   "Version"
   "WatchFilePattern"
   "AbortOnError"
   "AlphaBlending"
   "Antialiasing"
   "ArrowHeadRadius"
   "ArrowStemLength"
   "ArrowStemRadius"
   "Axes"
   "AxesMikado"
   "AxesAutoPosition"
   "AxesForceValue"
   "AxesMaxX"
   "AxesMaxY"
   "AxesMaxZ"
   "AxesMinX"
   "AxesMinY"
   "AxesMinZ"
   "AxesTicsX"
   "AxesTicsY"
   "AxesTicsZ"
   "AxesValueMaxX"
   "AxesValueMaxY"
   "AxesValueMaxZ"
   "AxesValueMinX"
   "AxesValueMinY"
   "AxesValueMinZ"
   "BackgroundGradient"
   "BackgroundImage3D"
   "BackgroundImagePage"
   "BackgroundImagePositionX"
   "BackgroundImagePositionY"
   "BackgroundImageWidth"
   "BackgroundImageHeight"
   "BoundingBoxSize"
   "Camera"
   "CameraAperture"
   "CameraEyeSeparationRatio"
   "CameraFocalLengthRatio"
   "Clip0A"
   "Clip0B"
   "Clip0C"
   "Clip0D"
   "Clip1A"
   "Clip1B"
   "Clip1C"
   "Clip1D"
   "Clip2A"
   "Clip2B"
   "Clip2C"
   "Clip2D"
   "Clip3A"
   "Clip3B"
   "Clip3C"
   "Clip3D"
   "Clip4A"
   "Clip4B"
   "Clip4C"
   "Clip4D"
   "Clip5A"
   "Clip5B"
   "Clip5C"
   "Clip5D"
   "ClipFactor"
   "ClipOnlyDrawIntersectingVolume"
   "ClipOnlyVolume"
   "ClipPositionX"
   "ClipPositionY"
   "ClipWholeElements"
   "ColorScheme"
   "ConfirmOverwrite"
   "ContextPositionX"
   "ContextPositionY"
   "DetachedMenu"
   "DetachedProcess"
   "DisplayBorderFactor"
   "DoubleBuffer"
   "DrawBoundingBoxes"
   "ExpertMode"
   "ExtraPositionX"
   "ExtraPositionY"
   "ExtraHeight"
   "ExtraWidth"
   "FastRedraw"
   "FieldPositionX"
   "FieldPositionY"
   "FieldHeight"
   "FieldWidth"
   "FileChooserPositionX"
   "FileChooserPositionY"
   "FltkColorScheme"
   "FltkRefreshRate"
   "FontSize"
   "GraphicsFontSize"
   "GraphicsFontSizeTitle"
   "GraphicsHeight"
   "GraphicsPositionX"
   "GraphicsPositionY"
   "GraphicsWidth"
   "HighOrderToolsPositionX"
   "HighOrderToolsPositionY"
   "HighResolutionGraphics"
   "InitialModule"
   "InputScrolling"
   "Light0"
   "Light0X"
   "Light0Y"
   "Light0Z"
   "Light0W"
   "Light1"
   "Light1X"
   "Light1Y"
   "Light1Z"
   "Light1W"
   "Light2"
   "Light2X"
   "Light2Y"
   "Light2Z"
   "Light2W"
   "Light3"
   "Light3X"
   "Light3Y"
   "Light3Z"
   "Light3W"
   "Light4"
   "Light4X"
   "Light4Y"
   "Light4Z"
   "Light4W"
   "Light5"
   "Light5X"
   "Light5Y"
   "Light5Z"
   "Light5W"
   "LineWidth"
   "ManipulatorPositionX"
   "ManipulatorPositionY"
   "MaxX"
   "MaxY"
   "MaxZ"
   "MenuWidth"
   "MenuHeight"
   "MenuPositionX"
   "MenuPositionY"
   "MessageFontSize"
   "MessageHeight"
   "MinX"
   "MinY"
   "MinZ"
   "MouseHoverMeshes"
   "MouseSelection"
   "MouseInvertZoom"
   "NativeFileChooser"
   "NonModalWindows"
   "NoPopup"
   "NumThreads"
   "OptionsPositionX"
   "OptionsPositionY"
   "Orthographic"
   "PluginPositionX"
   "PluginPositionY"
   "PluginHeight"
   "PluginWidth"
   "PolygonOffsetAlwaysOn"
   "PolygonOffsetFactor"
   "PolygonOffsetUnits"
   "ProgressMeterStep"
   "QuadricSubdivisions"
   "RotationX"
   "RotationY"
   "RotationZ"
   "RotationCenterGravity"
   "RotationCenterX"
   "RotationCenterY"
   "RotationCenterZ"
   "SaveOptions"
   "SaveSession"
   "ScaleX"
   "ScaleY"
   "ScaleZ"
   "Shininess"
   "ShininessExponent"
   "ShowModuleMenu"
   "ShowOptionsOnStartup"
   "ShowMessagesOnStartup"
   "SmallAxes"
   "SmallAxesPositionX"
   "SmallAxesPositionY"
   "SmallAxesSize"
   "StatisticsPositionX"
   "StatisticsPositionY"
   "Stereo"
   "SystemMenuBar"
   "Terminal"
   "Tooltips"
   "Trackball"
   "TrackballHyperbolicSheet"
   "TrackballQuaternion0"
   "TrackballQuaternion1"
   "TrackballQuaternion2"
   "TrackballQuaternion3"
   "TranslationX"
   "TranslationY"
   "TranslationZ"
   "VectorType"
   "Verbosity"
   "VisibilityPositionX"
   "VisibilityPositionY"
   "ZoomFactor"
   "Background"
   "Foreground"
   "Text"
   "AmbientLight"
   "DiffuseLight"
   "SpecularLight"
   "ParameterCommand"
   "Parameter"
   "ParameterFirst"
   "ParameterLast"
   "ParameterSteps"
   "CompositeWindows"
   "DeleteTemporaryFiles"
   "EpsBestRoot"
   "EpsCompress"
   "EpsLineWidthFactor"
   "EpsOcclusionCulling"
   "EpsPointSizeFactor"
   "EpsPS3Shading"
   "EpsQuality"
   "Format"
   "GeoLabels"
   "GeoOnlyPhysicals"
   "GifDither"
   "GifInterlace"
   "GifSort"
   "GifTransparent"
   "Height"
   "JpegQuality"
   "JpegSmoothing"
   "PgfTwoDim"
   "PgfExportAxis"
   "PgfHorizontalBar"
   "PostElementary"
   "PostElement"
   "PostGamma"
   "PostEta"
   "PostSICN"
   "PostSIGE"
   "PostDisto"
   "TexAsEquation"
   "TexForceFontSize"
   "TexWidthInMm"
   "X3dCompatibility"
   "X3dPrecision"
   "X3dRemoveInnerBorders"
   "X3dTransparency"
   "X3dSurfaces"
   "X3dEdges"
   "X3dVertices"
   "Width"
   "PostProcessing"
   "Algorithm"
   "Algorithm3D"
   "AlgorithmSwitchOnFailure"
   "AngleSmoothNormals"
   "AngleToleranceFacetOverlap"
   "AnisoMax"
   "AllowSwapAngle"
   "BdfFieldFormat"
   "Binary"
   "BoundaryLayerFanElements"
   "CgnsImportOrder"
   "CgnsImportIgnoreBC"
   "CgnsImportIgnoreSolution"
   "CgnsConstructTopology"
   "CgnsExportCPEX0045"
   "CgnsExportStructured"
   "ColorCarousel"
   "CompoundClassify"
   "CompoundMeshSizeFactor"
   "CpuTime"
   "CreateTopologyMsh2"
   "DrawSkinOnly"
   "Dual"
   "ElementOrder"
   "Explode"
   "FirstElementTag"
   "FirstNodeTag"
   "FlexibleTransfinite"
   "Hexahedra"
   "HighOrderDistCAD"
   "HighOrderIterMax"
   "HighOrderNumLayers"
   "HighOrderOptimize"
   "HighOrderPassMax"
   "HighOrderPeriodic"
   "HighOrderPoissonRatio"
   "HighOrderSavePeriodic"
   "HighOrderPrimSurfMesh"
   "HighOrderThresholdMin"
   "HighOrderThresholdMax"
   "HighOrderFastCurvingNewAlgo"
   "HighOrderCurveOuterBL"
   "HighOrderMaxRho"
   "HighOrderMaxAngle"
   "HighOrderMaxInnerAngle"
   "IgnoreParametrization"
   "IgnorePeriodicity"
   "LabelSampling"
   "LcIntegrationPrecision"
   "LightLines"
   "Lines"
   "LineLabels"
   "MaxIterDelaunay3D"
   "MaxNumThreads1D"
   "MaxNumThreads2D"
   "MaxNumThreads3D"
   "MaxRetries"
   "MeshOnlyVisible"
   "MeshOnlyEmpty"
   "MeshSizeExtendFromBoundary"
   "MeshSizeFactor"
   "MeshSizeMin"
   "MeshSizeMax"
   "MeshSizeFromCurvature"
   "MeshSizeFromCurvatureIsotropic"
   "MeshSizeFromPoints"
   "MeshSizeFromParametricPoints"
   "MetisAlgorithm"
   "MetisEdgeMatching"
   "MetisMaxLoadImbalance"
   "MetisObjective"
   "MetisMinConn"
   "MetisRefinementAlgorithm"
   "MinimumCircleNodes"
   "MinimumCurveNodes"
   "MinimumElementsPerTwoPi"
   "MshFileVersion"
   "MedFileMinorVersion"
   "MedImportGroupsOfNodes"
   "MedSingleModel"
   "NbHexahedra"
   "NbNodes"
   "NbPartitions"
   "NbPrisms"
   "NbPyramids"
   "NbTrihedra"
   "NbQuadrangles"
   "NbTetrahedra"
   "NbTriangles"
   "NewtonConvergenceTestXYZ"
   "Nodes"
   "NodeLabels"
   "NodeSize"
   "NodeType"
   "OldInitialDelaunay2D"
   "Optimize"
   "OptimizeThreshold"
   "OptimizeNetgen"
   "PartitionHexWeight"
   "PartitionLineWeight"
   "PartitionPrismWeight"
   "PartitionPyramidWeight"
   "PartitionQuadWeight"
   "PartitionTrihedronWeight"
   "PartitionTetWeight"
   "PartitionTriWeight"
   "PartitionCreateTopology"
   "PartitionCreatePhysicals"
   "PartitionCreateGhostCells"
   "PartitionSplitMeshFiles"
   "PartitionTopologyFile"
   "PartitionOldStyleMsh2"
   "PartitionConvertMsh2"
   "PreserveNumberingMsh2"
   "Prisms"
   "Pyramids"
   "QuadqsSizemapMethod"
   "QuadqsTopologyOptimizationMethods"
   "QuadqsRemeshingBoldness"
   "QuadqsScalingOnTriangulation"
   "Quadrangles"
   "QualityInf"
   "QualitySup"
   "QualityType"
   "RadiusInf"
   "RadiusSup"
   "RandomFactor"
   "RandomFactor3D"
   "RandomSeed"
   "ReadGroupsOfElements"
   "RecombinationAlgorithm"
   "RecombineAll"
   "RecombineOptimizeTopology"
   "Recombine3DAll"
   "Recombine3DLevel"
   "Recombine3DConformity"
   "RefineSteps"
   "Renumber"
   "ReparamMaxTriangles"
   "SaveAll"
   "SaveElementTagType"
   "SaveTopology"
   "SaveParametric"
   "SaveGroupsOfElements"
   "SaveGroupsOfNodes"
   "SecondOrderIncomplete"
   "SecondOrderLinear"
   "Smoothing"
   "SmoothCrossField"
   "CrossFieldClosestPoint"
   "SmoothNormals"
   "SmoothRatio"
   "StlAngularDeflection"
   "StlLinearDeflection"
   "StlOneSolidPerSurface"
   "StlRemoveDuplicateTriangles"
   "SubdivisionAlgorithm"
   "SurfaceEdges"
   "SurfaceFaces"
   "SwitchElementTags"
   "Tetrahedra"
   "ToleranceEdgeLength"
   "ToleranceInitialDelaunay"
   "ToleranceReferenceElement"
   "Triangles"
   "Trihedra"
   "UnvStrictFormat"
   "VolumeEdges"
   "VolumeFaces"
   "Voronoi"
   "ZoneDefinition"
   "NodesSup"
   "Zero"
   "One"
   "Two"
   "Three"
   "Four"
   "Five"
   "Six"
   "Seven"
   "Eight"
   "Nine"
   "Ten"
   "Eleven"
   "Twelve"
   "Thirteen"
   "Fourteen"
   "Fifteen"
   "Sixteen"
   "Seventeen"
   "Eighteen"
   "Nineteen"
   "Executable0"
   "Executable1"
   "Executable2"
   "Executable3"
   "Executable4"
   "Executable5"
   "Executable6"
   "Executable7"
   "Executable8"
   "Executable9"
   "Name0"
   "Name1"
   "Name2"
   "Name3"
   "Name4"
   "Name5"
   "Name6"
   "Name7"
   "Name8"
   "Name9"
   "Extension0"
   "Extension1"
   "Extension2"
   "Extension3"
   "Extension4"
   "Extension5"
   "Extension6"
   "Extension7"
   "Extension8"
   "Extension9"
   "OctaveInterpreter"
   "PythonInterpreter"
   "RemoteLogin0"
   "RemoteLogin1"
   "RemoteLogin2"
   "RemoteLogin3"
   "RemoteLogin4"
   "RemoteLogin5"
   "RemoteLogin6"
   "RemoteLogin7"
   "RemoteLogin8"
   "RemoteLogin9"
   "SocketName"
   "AlwaysListen"
   "AutoArchiveOutputFiles"
   "AutoCheck"
   "AutoLoadDatabase"
   "AutoSaveDatabase"
   "AutoMesh"
   "AutoMergeFile"
   "AutoShowViews"
   "AutoShowLastStep"
   "Plugins"
   "ShowInvisibleParameters"
   "Timeout"
   "DoubleClickedGraphPointCommand"
   "GraphPointCommand"
   "AnimationDelay"
   "AnimationCycle"
   "AnimationStep"
   "CombineRemoveOriginal"
   "CombineCopyOptions"
   "DoubleClickedGraphPointX"
   "DoubleClickedGraphPointY"
   "DoubleClickedView"
   "ForceElementData"
   "ForceNodeData"
   "GraphPointX"
   "GraphPointY"
   "HorizontalScales"
   "Link"
   "NbViews"
   "SaveInterpolationMatrices"
   "SaveMesh"
   "Attributes"
   "DoubleClickedCommand"
   "GeneralizedRaiseX"
   "GeneralizedRaiseY"
   "GeneralizedRaiseZ"
   "Group"
   "Name"
   "Stipple0"
   "Stipple1"
   "Stipple2"
   "Stipple3"
   "Stipple4"
   "Stipple5"
   "Stipple6"
   "Stipple7"
   "Stipple8"
   "Stipple9"
   "AbscissaRangeType"
   "AdaptVisualizationGrid"
   "ArrowSizeMax"
   "ArrowSizeMin"
   "AutoPosition"
   "Boundary"
   "CenterGlyphs"
   "Closed"
   "ColormapAlpha"
   "ColormapAlphaPower"
   "ColormapBeta"
   "ColormapBias"
   "ColormapCurvature"
   "ColormapInvert"
   "ColormapNumber"
   "ColormapRotation"
   "ColormapSwap"
   "ComponentMap0"
   "ComponentMap1"
   "ComponentMap2"
   "ComponentMap3"
   "ComponentMap4"
   "ComponentMap5"
   "ComponentMap6"
   "ComponentMap7"
   "ComponentMap8"
   "CustomAbscissaMax"
   "CustomAbscissaMin"
   "CustomMax"
   "CustomMin"
   "DisplacementFactor"
   "DrawHexahedra"
   "DrawLines"
   "DrawPoints"
   "DrawPrisms"
   "DrawPyramids"
   "DrawTrihedra"
   "DrawQuadrangles"
   "DrawScalars"
   "DrawStrings"
   "DrawTensors"
   "DrawTetrahedra"
   "DrawTriangles"
   "DrawVectors"
   "ExternalView"
   "FakeTransparency"
   "ForceNumComponents"
   "GeneralizedRaiseFactor"
   "GeneralizedRaiseView"
   "GlyphLocation"
   "IntervalsType"
   "LineType"
   "MaxRecursionLevel"
   "Max"
   "MaxVisible"
   "Min"
   "MinVisible"
   "NbIso"
   "NbTimeStep"
   "NormalRaise"
   "PositionX"
   "PositionY"
   "RaiseX"
   "RaiseY"
   "RaiseZ"
   "RangeType"
   "Sampling"
   "SaturateValues"
   "ScaleType"
   "ShowElement"
   "ShowScale"
   "ShowTime"
   "Stipple"
   "TargetError"
   "TensorType"
   "TimeStep"
   "Time"
   "Type"
   "UseGeneralizedRaise"
   "Visible"
   "Text2D"
   "Text3D"
   "PointNumbers"
   "Background2D"
   "ColorTable"
   "Solver")
  "List of GMSH constants.")

(defvar gmsh-font-lock-keywords
  (list
   (cons "!" 'font-lock-negation-char-face)
   (cons "\\bMesh\\b" 'font-lock-constant-face) ; increase priority for this constant
   (list "\\(Macro\\|Call\\)\\s-+\\(\\(?:\\sw\\|\\s_\\)+\\)\\b"
	 '(1 font-lock-keyword-face)
	 '(2 font-lock-function-name-face))
   (cons (concat "\\b" (regexp-opt gmsh-fontlock-keywords) "\\b") 'font-lock-keyword-face)
   (cons (concat "\\b" (regexp-opt gmsh-fontlock-builtin-functions) "\\b") 'font-lock-builtin-face)
   (cons (concat "\\b" (regexp-opt gmsh-fontlock-constants) "\\b") 'font-lock-constant-face)
   (list "\\(?:^\\|;\\)\\s-*\\(\\(?:\\sw\\|\\s_\\)+\\)\\(?:\\[\\]\\)?\\s-*[\\+-\\*/]?="
	 1 'font-lock-variable-name-face)))

(defvar gmsh-mode-map
  (let ((map (make-sparse-keymap)))
    ;; currently empty
    map)
  "GMSH keymap.")

(defvar gmsh--completion-keywords
  (delete-dups (mapcan #'split-string (append gmsh-fontlock-builtin-functions
                                              gmsh-fontlock-keywords
                                              gmsh-fontlock-constants)))
  "GMSH completion list.")

;;;###autoload
(defun gmsh-mode-completion-at-point ()
  "GMSH function to be used for the hook `completion-at-point-functions'."
  (interactive)
  (let* ((bds (bounds-of-thing-at-point 'symbol))
         (start (car bds))
         (end (cdr bds)))
    (list start end gmsh--completion-keywords . nil)))

;;;###autoload
(define-derived-mode gmsh-mode c-mode "GMSH"
  "GMSH mesh generator major mode."
  :group 'gmsh
  :syntax-table c-mode-syntax-table
  (setq-local comment-start "//"
              comment-end ""
              comment-padding 1
              indent-line-function 'c-indent-line
              font-lock-defaults '(gmsh-font-lock-keywords nil t))
  (add-hook 'completion-at-point-functions 'gmsh-mode-completion-at-point nil 'local))

;;;###autoload
(add-to-list 'auto-mode-alist '("\\.gmsh\\'" . gmsh-mode))
;;;###autoload
(add-to-list 'auto-mode-alist '("\\.geo\\'" . gmsh-mode))

(provide 'gmsh-mode)

;;; gmsh-mode.el ends here
